function [labels,clust_centers,cost] = manifold_kmeans_grassmann(points,N_clusters,max_iter)


%%%%%% points is a cell array of tall-thin orthonormal matrices.
%%%%% N_clusters is the number of clusters required.
%%%%% max_iter is the max no. of iterations for k-means

Npts = length(points);

rand_pts = randperm(Npts);

clust_idx = rand_pts(1:N_clusters);

clust_centers = points([clust_idx]);
labels = [];


cur_iter = 1;

while(cur_iter <=max_iter)
    
    %%%%% Assign each point to a cluster
    for i = 1:Npts
        
        clust_dists = zeros(1,N_clusters);
        
        for j = 1:N_clusters
            
                  clust_dists(j) = log_map_norm(points{i},clust_centers{j});
        end
        
        closest = find(clust_dists == min(clust_dists));
        
        labels(i) = closest;
    end
    
    
     %%%%% Compute clustering cost
    
    cost = 0;
    for i = 1:N_clusters
        
        clust_members = find(labels == i);
        
        clust_points = points([clust_members]);
        temp = 0;
        
        
        for j = 1:length(clust_points)
            
            temp  = temp + log_map_norm(clust_centers{i},clust_points{j});
        end
        
        
        cost = cost + temp;
        
    end
    
    %cost
    %%%%%% Now recompute cluster centers
    
    n_iter_karcher = 10;   %%%% this is tunable and needs experimentation
    
    for i = 1:N_clusters
        
        clust_members = find(labels == i);
        
        clust_points = points([clust_members]);
        
        clust_center = compute_grassmann_mean(clust_points,n_iter_karcher);
        
        clust_centers{i} = clust_center;
    end
    
    cur_iter = cur_iter + 1;
    
   
    
end

return;










